'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var command = _interopDefault(require('@pagedip/lib-command'));
var inquirer = _interopDefault(require('inquirer'));

var name = "pagedip";

var version = "5.2.0-rc.0";

var help = "Usage: pagedip [--help] [--version]\n\n  A Pagedip CLI tool.\n\nOptions\n  -h, -H, --help          Shows this message.\n  -v, -V, --version       Prints the version and exits.";

var runCommands = ["api", "auth", "couchdb-setup", "live", "setup", "theme", "writeroom"];

var asyncToGenerator = function (fn) {
  return function () {
    var gen = fn.apply(this, arguments);
    return new Promise(function (resolve, reject) {
      function step(key, arg) {
        try {
          var info = gen[key](arg);
          var value = info.value;
        } catch (error) {
          reject(error);
          return;
        }

        if (info.done) {
          resolve(value);
        } else {
          return Promise.resolve(value).then(function (value) {
            step("next", value);
          }, function (err) {
            step("throw", err);
          });
        }
      }

      return step("next");
    });
  };
};

var cli = command("pagedip", {
  module, help,
  version: `${name} ${version}`
}, (() => {
  var _ref = asyncToGenerator(function* (args, config, logger) {
    switch (args[0]) {
      case "install":
        yield this.run("pagedip-install", args.slice(1), logger);
        break;

      case "run":
        yield this.run("pagedip-run", args.slice(1), logger);
        break;

      default:
        if (runCommands.includes(args[0])) {
          yield this.run("pagedip-run", args, logger);
          break;
        }

        logger.info(help);
    }
  });

  return function (_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
})());

command("pagedip-install", {}, (() => {
  var _ref2 = asyncToGenerator(function* (args, config, logger) {
    yield require("./").install(args, { logger });
  });

  return function (_x4, _x5, _x6) {
    return _ref2.apply(this, arguments);
  };
})());

command("pagedip-run", {}, (() => {
  var _ref3 = asyncToGenerator(function* (args, config, logger) {
    const { install, load } = require("./");
    let cmd;

    try {
      cmd = yield load(args[0], { logger });
    } catch (e) {
      if (e.code !== "MODULE_NOT_FOUND") throw e;

      const { confirm } = yield inquirer.prompt([{
        type: "confirm",
        name: "confirm",
        message: `Could not locate a package named '${args[0]}'. Do you want to attempt to install it?`,
        default: true
      }]);

      if (!confirm) throw e;

      yield install(args[0], { logger });
      cmd = yield load(args[0], { logger });
    }

    yield this.run(cmd, args.slice(1), logger);
  });

  return function (_x7, _x8, _x9) {
    return _ref3.apply(this, arguments);
  };
})());

module.exports = cli;
