'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var child_process = require('child_process');
var getNpmCliPath = _interopDefault(require('npm-cli-path'));
var resolve = require('@pagedip/util-resolve');
var resolve__default = _interopDefault(resolve);
var path = require('path');
var findUp = _interopDefault(require('find-up'));

var asyncToGenerator = function (fn) {
  return function () {
    var gen = fn.apply(this, arguments);
    return new Promise(function (resolve$$1, reject) {
      function step(key, arg) {
        try {
          var info = gen[key](arg);
          var value = info.value;
        } catch (error) {
          reject(error);
          return;
        }

        if (info.done) {
          resolve$$1(value);
        } else {
          return Promise.resolve(value).then(function (value) {
            step("next", value);
          }, function (err) {
            step("throw", err);
          });
        }
      }

      return step("next");
    });
  };
};











var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

function formatPackageName(pkg) {
  if (typeof pkg !== "string" || !pkg) {
    throw new Error("Expecting non-empty string for package name.");
  }

  if (!resolve.isPath.test(pkg) && pkg.substr(0, 9) !== "@pagedip/") {
    pkg = "@pagedip/" + pkg;
  }

  return pkg;
}

let npmCli;
let npmPath = (() => {
  var _ref = asyncToGenerator(function* () {
    if (npmCli) return npmCli;
    const p = yield getNpmCliPath();
    npmCli = p;
    return p;
  });

  return function npmPath() {
    return _ref.apply(this, arguments);
  };
})();

let npmCommand = (() => {
  var _ref2 = asyncToGenerator(function* (args) {
    const cli = yield npmPath();

    return yield new Promise(function (resolve$$1, reject) {
      const cp = child_process.spawn("node", [cli, ...args], {
        stdio: "inherit"
      });

      cp.on("exit", function (code) {
        code ? reject(new Error("npm command failed")) : resolve$$1();
      });

      cp.on("error", reject);
    });
  });

  return function npmCommand(_x) {
    return _ref2.apply(this, arguments);
  };
})();

function npmInstall(pkgname) {
  return npmCommand(["i", pkgname, "-g"]);
}

var install = (() => {
  var _ref = asyncToGenerator(function* (pkgs, opts) {
    if (!Array.isArray(pkgs)) pkgs = pkgs == null ? [] : [pkgs];

    opts = opts || {};

    for (let i = 0; i < pkgs.length; i++) {
      if (opts.logger) opts.logger.debug("Installing package '%s'", pkgs[i]);
      yield npmInstall(formatPackageName(pkgs[i]));
    }
  });

  function install(_x, _x2) {
    return _ref.apply(this, arguments);
  }

  return install;
})();

let lerna;

try {
  lerna = require("lerna");
} catch (e) {
  // eat errors
}

let resolveCli = (() => {
  var _ref = asyncToGenerator(function* (pkgname, opts) {
    lernaif: if (lerna) {
      const lernafile = yield findUp("lerna.json", { cwd: __dirname });
      if (!lernafile) break lernaif;

      const ls = new lerna.LsCommand(["ls"], {}, path.dirname(lernafile));
      ls.runValidations();
      ls.runPreparations();
      yield new Promise(function (r) {
        return ls._attempt("initialize", r);
      });

      const lpkgname = formatPackageName(pkgname);
      const pkg = ls.packages.find(function ({ name }) {
        return name === lpkgname;
      });
      if (!pkg) break lernaif;

      pkgname = pkg.location;
    }

    return yield resolve__default(pkgname, _extends({}, opts, {
      leads: ["@pagedip/", ""],
      packageFilter(pkg) {
        if (pkg.command) pkg.main = pkg.command;else delete pkg.main;
        return pkg;
      }
    }));
  });

  return function resolveCli(_x, _x2) {
    return _ref.apply(this, arguments);
  };
})();

var load = (() => {
  var _ref2 = asyncToGenerator(function* (pkgname, opts) {
    const cliFile = yield resolveCli(pkgname, opts);

    if (!cliFile) {
      const err = new Error(`Could not find package '${pkgname}'.`);
      err.code = "MODULE_NOT_FOUND";
      throw err;
    }

    return require(cliFile);
  });

  function load(_x3, _x4) {
    return _ref2.apply(this, arguments);
  }

  return load;
})();

exports.install = install;
exports.load = load;
